!	CFSPML.f90 
!
!	FUNCTIONS:
!		CFSPML - Implement Gedney's convolution CFS Perfectly Matched Layer.
!
!		In various PML profiles and region sizes.
!

!****************************************************************************
!
!	PROGRAM:	CFSPML
!
!	PURPOSE:	Implement Gedney's convolution CFS.
!	
!	AUTHOR:		Kristen Parrish, UT Austin EE (kristen.n.parrish@gmail.com)
!				Chicheng Xu, UT Austin PGE (xuchicheng@mail.utexas.edu)
!
!	DATE:		Mar 23, 2009
!
!****************************************************************************

	program CFSPML

	implicit none

	!define math & physics constants
	double precision, parameter :: pi	= 4.0d0*atan(1.0d0)
	double precision, parameter :: u0   = 4.0d-7 * pi	!vacuum permeability
	double precision, parameter :: c0   = 299792458.0d0	!speed of light in vacuum
	double precision, parameter :: eps0 = 1.0d0/(u0*c0**2) !vacuum permitivity
	double precision, parameter :: epsr = 1.0d0			!relative permitivity
	double precision, parameter :: ur	= 1.0d0			!relative permeability
	double precision, parameter :: alphax = 0.05d0		!better scan from CompSettings file
	double precision, parameter :: alphay = 0.05d0		!better scan from CompSettings file
	
	!allocate memory for E and H fields, E at the cross point
	double precision, allocatable :: Et(:)				!sample Ez in one fixed point
	double precision, allocatable :: Ez(:,:)	
	double precision, allocatable :: Hx(:,:)
	double precision, allocatable :: Hy(:,:)

	double precision, allocatable :: phi_ezx(:,:)
	double precision, allocatable :: phi_ezy(:,:)
	double precision, allocatable :: phi_hxy(:,:)
	double precision, allocatable :: phi_hyx(:,:)
	
	double precision, allocatable :: sigma(:)	
	double precision, allocatable :: kx(:)		
	double precision, allocatable :: ky(:)	
	double precision, allocatable :: bx(:)	
	double precision, allocatable :: by(:)
	double precision, allocatable :: ax(:)	
	double precision, allocatable :: ay(:)  		


	!initialize computing parameters
	double precision lx, ly, m, dt_factor
	integer nx, ny, nt, npml
	double precision dx, dy, dt, t0, tw, omega, sigmaMax
	double precision ax_t, ay_t, bx_t, by_t, kx_t, ky_t	!true values used in grids
	integer nx0, ny0, n, i, j, expNo
	double precision c1, c2, c3, c4, c5, c6	!common coefficients
	real t_start, t_stop
	character(len=10)  sourceType
	character(len=30) fileName
	character(len=3) expNoStr
	integer continueRead

	!delete all old files in a new experiment
	call system('del CFS*.txt')

	!read computing parameters from external file
	open(unit=100, file = 'CompSettings.txt')

	!skip the first head line
	read(unit=100, fmt='(1X,A10)') sourceType

	expNo = 0
	continueRead = 1

	do while (continueRead == 1)		
		read(unit=100, fmt='(1X,A10,F10.2,F10.2,I10,I10,I10,I10,F10.2,F10.2)') &
		sourceType, lx, ly, nx, ny, nt, npml, m, dt_factor
		
		if ( nx /= 0 )	then	!not reach the end of experiments
			call cpu_time  ( t_start)

			expNo = expNo + 1

			nx0 = nx/2 + 1		!center point of x direction
			ny0 = ny/2 + 1		!center point of y direction
			dx = lx / real(nx)      !grid size along x
			dy = ly / real(ny)      !grid size along y
			dt = dt_factor * sqrt(dx**2 + dy**2)/(2*c0) !magic time step * factor
			omega = pi/(20.0d0*dt)	!frequency of time harmonic source
			
			!max sigma in PML, see Eqn. 38 in Gedney's paper 
			!sigmaMax = (m+1.0d0)/(150.0d0*pi*dx*sqrt(epsr))			
			sigmaMax = 90
			!allocate array dimensions
			allocate(Et(nt))
			allocate(Ez(nx+1,ny+1))
			allocate(Hx(nx+1,ny))
			allocate(Hy(nx,ny+1))
			allocate(sigma(2*npml+1))		
			
			allocate(phi_ezx(nx+1,ny+1))
			allocate(phi_ezy(nx+1,ny+1))
			allocate(phi_hxy(nx+1,ny))
			allocate(phi_hyx(nx,ny+1))
			allocate(ax(2*npml+1))															
			allocate(ay(2*npml+1))														
			allocate(bx(2*npml+1))	
			allocate(by(2*npml+1))	
			allocate(kx(2*npml+1))	
			allocate(ky(2*npml+1))	

			!initialize all fields as zero
			Et = 0
			Ez = 0
			Hx = 0
			Hy = 0
			sigma = 0
			phi_hyx = 0
			phi_hxy = 0
			phi_ezx = 0
			phi_ezy = 0
			
			!initiate the guassian pulse current source
			t0 = 60.0d0           ! The center of the Gaussian Pulse in term of dt
			tw = 10.0d0           ! The width of the Gaussian Pulse in term of dt

			n = 0
			
			!figure source type to use
			if (sourceType == ' Gaussian ') then
				Ez(nx0,ny0) = 1000.0d0*exp(-(n-t0)**2/(tw**2)) !(dt/eps0)*J(nx0,ny0) at t=0
			else if (sourceType == ' Harmonic ') then
				Ez(nx0,ny0) = 1000.0d0*cos(omega*n*dt); !(dt/eps0)*J(nx0,ny0) at t=0
			else
				print *, 'Wrong source type!'
			end if

			
			!calculate the conductivities in PML, same profile for x & y directions
			do i=1, 2*npml+1
				sigma(i) = sigmaMax * ((i-1)/(2.0d0*npml))**m
				
				!TBD: kx and ky's profile, see Gedney's paper
				kx(i) = 1.0d0 + 10.0d0 * ((i-1)/(2.0d0*npml))**m
				ky(i) = 1.0d0 + 10.0d0 * ((i-1)/(2.0d0*npml))**m
				
				bx(i) = exp(-1.0d0*(sigma(i)/kx(i)+alphax)*(dt/eps0))
				by(i) = exp(-1.0d0*(sigma(i)/ky(i)+alphay)*(dt/eps0))

				ax(i) = sigma(i)*(bx(i)-1.0d0)/ (sigma(i)*kx(i) + alphax*kx(i)**2)
				ay(i) = sigma(i)*(by(i)-1.0d0)/ (sigma(i)*ky(i) + alphay*ky(i)**2)
			end do

			!calculate some common coefficients to enhance performance
			c1 = dt/(u0*dy)
			c2 = dt/(u0*dx)
			c3 = dt/(eps0*dy)
			c4 = dt/(eps0*dx) 
			c5 = dt/(u0*ur)
			c6 = dt/(eps0*epsr)

			!iterate time step
			do n=1,nt				
				!********************************************************************
				!update Hx in vacuum space, including the vacuum-PML interface
				do j=npml+1, ny-npml
					Hx(npml+1:nx-npml, j) = Hx(npml+1:nx-npml, j) -					&
					c1*(Ez(npml+1:nx-npml,j+1)-Ez(npml+1:nx-npml,j))
				end do
				
				!update Hx at the bottom PML region I + two bottom regions III
				do j=1,npml
					by_t = by(2*(npml+1-j))
					ay_t = ay(2*(npml+1-j))
					ky_t = ky(2*(npml+1-j))
					
					phi_hxy(:,j)=by_t*phi_hxy(:,j)+	ay_t*(Ez(:,j+1)-Ez(:,j))/dy					
					
					Hx(:,j) = Hx(:,j) - c5*((Ez(:,j+1)-	Ez(:,j))/(dy*ky_t) + phi_hxy(:,j))	
				end do

				!update Hx at the top PML region I + two top regions III 
				do j=ny-npml+1,ny
					by_t = by(2*(j-ny+npml))
					ay_t = ay(2*(j-ny+npml))
					ky_t = ky(2*(j-ny+npml))
					
					phi_hxy(:,j)=by_t*phi_hxy(:,j) + ay_t*(Ez(:,j+1)-Ez(:,j))/dy					
					
					Hx(:,j) = Hx(:,j)-	c5*((Ez(:,j+1)-	Ez(:,j))/(dy*ky_t) + phi_hxy(:,j))	
				end do
									
				!update Hx at the left PML region II, phi_hxy=0
				do j = npml+1, ny-npml
					ky_t = 1.0;
								
					Hx(1:npml,j) = Hx(1:npml,j)- c5*(Ez(1:npml,j+1)- Ez(1:npml,j))/(dy*ky_t)
				end do
			
				!update Hx at the right PML region II, phi_hxy=0
				do j = npml+1,ny-npml
					ky_t = 1.0
								
					Hx(nx-npml+1:nx+1,j) = Hx(nx-npml+1:nx+1,j)-				&
					c5*(Ez(nx-npml+1:nx+1,j+1)-	Ez(nx-npml+1:nx+1,j))/(dy*ky_t)
				end do

				!Due to PEC frame
				Hx(1,:)=0
				Hx(nx+1,:)=0				
				!********************************************************************

				!********************************************************************
				!update Hy in vacuum space where sigma = 0
				do i=npml+1, nx-npml
					Hy(i, npml+1:ny-npml) = Hy(i, npml+1:ny-npml) +					&
					c2*(Ez(i+1,npml+1:ny-npml)-Ez(i,npml+1:ny-npml))
				end do

				!update Hy at the top PML region I, phi_hyx = 0
				do i=npml+1,nx-npml
					kx_t = 1.0
									
					Hy(i,ny-npml+1:ny+1) = Hy(i,ny-npml+1:ny+1)+					&
					c5*(Ez(i+1,ny-npml+1:ny+1)-	Ez(i,ny-npml+1:ny+1))/(dx*kx_t)
				end do			

				!update Hy at the bottom PML region I, phi_hyx = 0
				do i=npml+1,nx-npml
					kx_t = 1.0
									
					Hy(i,1:npml) = Hy(i,1:npml)+c5*(Ez(i+1,1:npml) - Ez(i,1:npml))/(dx*kx_t)
				end do
			
				!update Hy at the left PML region II + two left corners
				do i=1,npml					
					bx_t = bx(2*(npml+1-i))
					ax_t = ax(2*(npml+1-i))
					kx_t = kx(2*(npml+1-i))
					
					phi_hyx(i,:)=bx_t*phi_hyx(i,:)+ax_t*(Ez(i+1,:)-Ez(i,:))/dx
					
					Hy(i,:) = Hy(i,:) + c5*((Ez(i+1,:)-Ez(i,:))/(dx*kx_t)+phi_hyx(i,:))	
				end do

				!update Hy at the right PML region II + two right corners
				do i=nx-npml+1,nx
					bx_t = bx(2*(i-nx+npml))
					ax_t = ax(2*(i-nx+npml))
					kx_t = kx(2*(i-nx+npml))

					phi_hyx(i,:)=bx_t*phi_hyx(i,:)+	ax_t*(Ez(i+1,:)-Ez(i,:))/dx
					
					Hy(i,:) = Hy(i,:) + c5*((Ez(i+1,:)-Ez(i,:))/(dx*kx_t)+phi_hyx(i,:))
				end do

				!due to PEC frame
				Hy(:, 1)=0
				Hy(:, ny+1)=0
				!********************************************************************

				!********************************************************************
				!update Ez in vacuum space without current source
				do i=npml+1, nx+1-npml
					do j = npml+1, ny+1-npml
						Ez(i,j) = Ez(i,j)+ c4*(Hy(i,j)-Hy(i-1,j)) -					&
						c3*(Hx(i,j)-Hx(i,j-1))
					end do
				end do

				!apply source in the center; source term = (dt/eps0)*J(nx0,ny0, t)			
				if (sourceType == ' Gaussian ') then
					Ez (nx0,ny0) = Ez(nx0,ny0) + 1000.0d0*exp(-(n-t0)**2/tw**2); 
					!Ez(nx0,ny0) = 1000*exp(-(n-t0)**2/tw**2); !hard source
				else if (sourceType == ' Harmonic ') then
					Ez(nx0,ny0) = Ez(nx0,ny0) + 1000.0d0*cos(omega*n*dt);
					!Ez(nx0,ny0) = 1000.0 * cos(omega*n*dt); !hard source
				else
					print *, 'Wrong source type!'
				end if

				!update Ez at the top PML region I, phi_ezx = 0
				do j=ny-npml+2, ny					
					by_t = by(2*(j-ny+npml)-1)
					ay_t = ay(2*(j-ny+npml)-1)
					ky_t = ky(2*(j-ny+npml)-1)				
				
					phi_ezy(npml+1:nx-npml+1,j) = by_t*phi_ezy(npml+1:nx-npml+1,j) +	&
					ay_t*(Hx(npml+1:nx-npml+1,j)-Hx(npml+1:nx-npml+1,j-1))/dy
					
					Ez(npml+1:nx-npml+1,j) = Ez(npml+1:nx-npml+1,j) -				&
					c6*((Hx(npml+1:nx-npml+1,j)-Hx(npml+1:nx-npml+1,j-1))/(dy*ky_t)	&
					+phi_ezy(npml+1:nx-npml+1,j))
				end do
				

				!update Ez at the bottom PML region I, phi_ezx = 0
				do j=2,npml					
					by_t = by(2*(npml+1-j)+1)
					ay_t = ay(2*(npml+1-j)+1)
					ky_t = ky(2*(npml+1-j)+1)

					phi_ezy(npml+1:nx-npml+1,j) = by_t*phi_ezy(npml+1:nx-npml+1,j) +	&
					ay_t*(Hx(npml+1:nx-npml+1,j)-Hx(npml+1:nx-npml+1,j-1))/dy
					
					Ez(npml+1:nx-npml+1,j) = Ez(npml+1:nx-npml+1,j) -				&
					c6*((Hx(npml+1:nx-npml+1,j)-Hx(npml+1:nx-npml+1,j-1))/(dy*ky_t)	&
					+phi_ezy(npml+1:nx-npml+1,j))
				end do
			
				!update Ez at the left PML region II, phi_ezy = 0
				do i=2,npml					
					bx_t = bx(2*(npml+1-i)+1)
					ax_t = ax(2*(npml+1-i)+1)
					kx_t = kx(2*(npml+1-i)+1)		

					phi_ezx(i,npml+1:ny-npml+1)=bx_t*phi_ezx(i,npml+1:ny-npml+1)		&
					+ax_t*(Hy(i,npml+1:ny-npml+1)-Hy(i-1,npml+1:ny-npml+1))/dx					
		
					Ez(i, npml+1:ny-npml+1) = Ez(i,npml+1:ny-npml+1) +				&
					c6*((Hy(i,npml+1:ny-npml+1)-Hy(i-1,npml+1:ny-npml+1))/(dx*kx_t)&
					+ phi_ezx(i,npml+1:ny-npml+1))
				end do

				!update Ez at the right PML region II, phi_ezy = 0
				do i=nx-npml+2,nx					
					bx_t = bx(2*(i-nx+npml)-1)
					ax_t = ax(2*(i-nx+npml)-1)
					kx_t = kx(2*(i-nx+npml)-1)

					phi_ezx(i,npml+1:ny-npml+1)=bx_t*phi_ezx(i,npml+1:ny-npml+1)		&
					+ax_t*(Hy(i,npml+1:ny-npml+1)-Hy(i-1,npml+1:ny-npml+1))/dx					
		
					Ez(i, npml+1:ny-npml+1) = Ez(i,npml+1:ny-npml+1) +				&
					c6*((Hy(i,npml+1:ny-npml+1)-Hy(i-1,npml+1:ny-npml+1))/(dx*kx_t)	&
					+ phi_ezx(i,npml+1:ny-npml+1))
				end do

				!update Ez at the bottom-left PML region III
				do i=2,npml					
					bx_t = bx(2*(npml+1-i)+1)					
					ax_t = ax(2*(npml+1-i)+1)
					kx_t = kx(2*(npml+1-i)+1)					
					
					do j=2,npml
						by_t = by(2*(npml+1-j)+1)
						ay_t = ax(2*(npml+1-j)+1)
						ky_t = ky(2*(npml+1-j)+1)

						phi_ezy(i,j)=by_t*phi_ezy(i,j)+ay_t*(Hx(i,j)-Hx(i,j-1))/dy
						phi_ezx(i,j)=bx_t*phi_ezx(i,j)+ax_t*(Hy(i,j)-Hy(i-1,j))/dx
						
						Ez(i,j) = Ez(i,j) + c6*((Hy(i,j)-Hy(i-1,j))/(dx*kx_t)			&
						-(Hx(i,j)-Hx(i,j-1))/(dy*ky_t) + phi_ezx(i,j)- phi_ezy(i,j))
					end do
				end do

				!update Ez at the bottom-right PML region III
				do i=nx-npml+2,nx
					bx_t = bx(2*(i-nx+npml)-1)					
					ax_t = ax(2*(i-nx+npml)-1)
					kx_t = kx(2*(i-nx+npml)-1)					
					
					do j=2,npml
						by_t = by(2*(npml+1-j)+1)
						ay_t = ax(2*(npml+1-j)+1)
						ky_t = ky(2*(npml+1-j)+1)

						phi_ezy(i,j)=by_t*phi_ezy(i,j)+ay_t*(Hx(i,j)-Hx(i,j-1))/dy
						phi_ezx(i,j)=bx_t*phi_ezx(i,j)+ax_t*(Hy(i,j)-Hy(i-1,j))/dx
						
						Ez(i,j) = Ez(i,j) + c6*((Hy(i,j)-Hy(i-1,j))/(dx*kx_t)			&
						-(Hx(i,j)-Hx(i,j-1))/(dy*ky_t) + phi_ezx(i,j)- phi_ezy(i,j))
					end do
				end do			
				
				!update Ez at the top-left PML region III
				do i=2,npml	
					ax_t = ax(2*(npml+1-i)+1)									
					bx_t = bx(2*(npml+1-i)+1)
					kx_t = kx(2*(npml+1-i)+1)
										
					do j=ny-npml+2, ny
						by_t = by(2*(j-ny+npml)-1)					
						ay_t = ay(2*(j-ny+npml)-1)
						ky_t = ky(2*(j-ny+npml)-1)

						phi_ezy(i,j)=by_t*phi_ezy(i,j)+ay_t*(Hx(i,j)-Hx(i,j-1))/dy
						phi_ezx(i,j)=bx_t*phi_ezx(i,j)+ax_t*(Hy(i,j)-Hy(i-1,j))/dx
						
						Ez(i,j) = Ez(i,j) + c6*((Hy(i,j)-Hy(i-1,j))/(dx*kx_t)			&
						-(Hx(i,j)-Hx(i,j-1))/(dy*ky_t) + phi_ezx(i,j)- phi_ezy(i,j))
					end do
				end do
				
				!update Ez at the top-right PML region III
				do i=nx-npml+2,nx
					bx_t = bx(2*(i-nx+npml)-1)					
					ax_t = ax(2*(i-nx+npml)-1)	
					kx_t = kx(2*(i-nx+npml)-1)
					
					do j=ny-npml+2, ny
						by_t = by(2*(j-ny+npml)-1)					
						ay_t = ay(2*(j-ny+npml)-1)
						ky_t = ky(2*(j-ny+npml)-1)	
												
						phi_ezy(i,j)=by_t*phi_ezy(i,j)+ay_t*(Hx(i,j)-Hx(i,j-1))/dy
						phi_ezx(i,j)=bx_t*phi_ezx(i,j)+ax_t*(Hy(i,j)-Hy(i-1,j))/dx
						
						Ez(i,j) = Ez(i,j) + c6*((Hy(i,j)-Hy(i-1,j))/(dx*kx_t)			&
						-(Hx(i,j)-Hx(i,j-1))/(dy*ky_t) + phi_ezx(i,j)- phi_ezy(i,j))
					end do
				end do
				
				!due to PEC frame
				Ez(1,:)=0
				Ez(nx+1,:)=0
				Ez(:,1)=0
				Ez(:,ny+1)=0

				!sample the electric field at chosen point
				!Et(n) = Ez(nx0+99,ny0+1);
				!********************************************************************
			end do


			!get elapse time
			call cpu_time ( t_stop )
			print *, 'Elapsed CPU time = ', t_stop - t_start


			!convert the experiment no. to string and use it in the output file name
			if ( expNo < 10 ) then
				write( expNoStr, '(i1)' )  expNo
				expNoStr = '00'//expNoStr
			else if ( expNo >= 10 .and. expNo <100 ) then
				write( expNoStr, '(i2)' )  expNo
				expNoStr = '0'//expNoStr
			else
				write( expNoStr, '(i3)' )  expNo
			end if
			
			!output the results
			fileName = 'CFS_ComputingTime.txt'
			open(unit=0,file=fileName, access='append')
			write(0, *) nt, t_stop - t_start
			close(0)  
				
			fileName = 'CFS_Exp'//expNoStr//'Ez.txt'
			open(unit=1,file=fileName) 	
			do, j=1,ny+1
				write (1,"(2000(1x,f0.10))") Ez(:,j)
			end do
			close(1)  

			fileName = 'CFS_Exp'//expNoStr//'Hx.txt'
			open(unit=2,file=fileName) 
			do, j=1,ny
				write (2,"(2000(1x,f0.10))") Hx(:,j)
			end do
			close(2)  

			fileName = 'CFS_Exp'//expNoStr//'Hy.txt'
			open(unit=3,file=fileName) 	
			do, j=1,ny+1
				write (3,"(2000(1x,f0.10))") Hy(:,j)
			end do
			close(3) 
			
			fileName = 'CFS_Exp'//expNoStr//'Sigma.txt'
			open(unit=4,file=fileName)
			write (4,"(2000(1x,f0.10))") sigma(:)
			close(4) 

			fileName = 'CFS_Exp'//expNoStr//'Et.txt'
			open(unit=5,file=fileName)
			write (5,"(2000(1x,f0.10))") Et(:)
			close(5) 

			deallocate(Et, Ez, Hx, Hy, sigma) 
			deallocate(phi_ezx,phi_ezy,phi_hxy,phi_hyx)	
			deallocate(ax,ay,bx,by, kx, ky)	
			
		else
			continueRead = 0	!end of experiments
		end if 

	end do

	close(100)

	end program CFSPML
