program splitfield_pml
  ! FDTD solver for infinite current source in free space, oriented in z-direction
  ! uses Yee Grid for vacuum, split-field PML formulation for outer boundary
  ! by Paul Tsuji, Computational Electromagnetics Group, UT-Austin
  implicit none
  real::dx,dy,dt,c,eps,mu,length,width,r
  real::tw,t_0,pi,Jz,eta2,c1,c2,c3,c4,omega
  real::sigx1,sigx2,sigy1,sigy2,sigmax,m,dt_factor
  real,allocatable::Ez(:,:),Hx(:,:),Hy(:,:),Et(:)
  real,allocatable::Ezx(:,:),Ezy(:,:),sig1(:),sig2(:)
  integer::nx,ny,nt,i,j,k,pml,continueRead,expNo,nxl,nyl
  real etime  
  real elapsed(2)    
  real total
  character(len=10)::sourceType
  character(len=30)::fileName
  character(len=3)::expNoStr

  open(unit=100,file='CompSettings.txt')
  read(unit=100,fmt='(1X,A10)') sourceType
  expNo = 0
  continueRead = 1
  ! set constants
  pi=4.0d0*atan(1.0d0); c=299792458.0d0 
  mu=4*pi*1.0d-7; eps=1.0d0/(mu*(c**2))

  do while (continueRead == 1)
     ! read computational parameters
     read(unit=100, fmt='(1X,A10,F10.2,F10.2,I10,I10,I10,I10,F10.2,F10.2)') &
     sourceType, width, length, nxl, nyl, nt, pml, m, dt_factor
     nx= 400
	 ny = 400
     if (m/=0) then
        expNo=expNo+1   !expNo - experiment number      
        dx=0.5/real(500); dy=0.5/real(500)
        dt=dt_factor*sqrt(dx**2+dy**2)/(2.0d0*c)
        t_0=60.0d0; tw=10.0d0
        omega=pi/(20.0d0*dt)
        ! max sigma for split-field pml, experimental reflection R(0)=exp(-16)
        sigmax = 90
		!sigmax=-(m+1.0d0)*(-16.0d0)*eps*c/(pml*dx)
        ! preprocessing: constants
        c1 = dt/(mu*dy)
        c2 = dt/(mu*dx)
        c3 = dt/(eps*dy)
        c4 = dt/(eps*dx)
        eta2=mu/eps
        allocate(Ez(nx-1,ny-1))
        allocate(Hx(nx-1,ny))
        allocate(Hy(nx,ny-1))
        allocate(Ezx(nx-1,ny-1))
        allocate(Ezy(nx-1,ny-1))
        allocate(Et(nt))
        allocate(sig1(pml))
        allocate(sig2(pml))
        ! setting pml profile
        do i=1,pml
           sig1(i)=sigmax*(real(i)/real(pml))**m
        end do
        do i=1,pml
           sig2(i)=sigmax*((real(i-1)+0.5d0)/real(pml))**m
        end do
        ! set initial conditions
        Ez=0.0d0; Hx=0.0d0; Hy=0.0d0
        Ezx=0.0d0; Ezy=0.0d0; Et=0.0d0
        ! set initial source condition
        Ezx(nx/2,ny/2)=Ezx(nx/2,ny/2)+500.0d0*exp(-(t_0)**2/tw**2)
        Ezy(nx/2,ny/2)=Ezy(nx/2,ny/2)+500.0d0*exp(-(t_0)**2/tw**2)
        do i=1,nt
           ! Update Hx where sigma_y=0 and Hy where sigma_x=0
           do j=pml+1,ny-pml
              Hx(:,j)=Hx(:,j)-c1*(Ez(:,j)-Ez(:,j-1))
           end do
           do j=pml+1,nx-pml
              Hy(j,:)=Hy(j,:)+c2*(Ez(j,:)-Ez(j-1,:))
           end do
           ! Update H-fields in PML regions
           do j=2,pml
              ! update Hx in bottom and top regions
              sigy2=sig1(pml-j+1)*eta2
              Hx(:,j)=exp(-sigy2*dt/mu)*Hx(:,j)-(1.0d0-exp(-sigy2*dt/mu))*(Ez(:,j)-Ez(:,j-1))/dy/sigy2
              Hx(:,ny-j+1)=exp(-sigy2*dt/mu)*Hx(:,ny-j+1)-(1.0d0-exp(-sigy2*dt/mu))*(Ez(:,ny-j+1)-Ez(:,ny-j))/dy/sigy2
              ! update Hy in two side regions
              sigx2=sig1(pml-j+1)*eta2
              Hy(j,:)=exp(-sigx2*dt/mu)*Hy(j,:)+(1.0d0-exp(-sigx2*dt/mu))*(Ez(j,:)-Ez(j-1,:))/dy/sigx2
              Hy(nx-j+1,:)=exp(-sigx2*dt/mu)*Hy(nx-j+1,:)+(1.0d0-exp(-sigx2*dt/mu))*(Ez(nx-j+1,:)-Ez(nx-j,:))/dx/sigx2
           end do
           ! Update Ezx where sigma_x=0 and Ezy where sigma_y=0
           do j=pml+1,nx-pml-1
              Ezx(j,:)=Ezx(j,:)+c4*(Hy(j+1,:)-Hy(j,:))
           end do
           do j=pml+1,ny-pml-1
              Ezy(:,j)=Ezy(:,j)-c3*(Hx(:,j+1)-Hx(:,j))
           end do
           ! Update Ezx, Ezy in PML regions
           do j=1,pml
              ! update Ezx in two side regions
              sigx1=sig2(pml-j+1)
              Ezx(j,:)=exp(-sigx1*dt/eps)*Ezx(j,:)+(1.0d0-exp(-sigx1*dt/eps))*((Hy(j+1,:)-Hy(j,:))/dx)/sigx1
              Ezx(nx-j,:)=exp(-sigx1*dt/eps)*Ezx(nx-j,:)+(1.0d0-exp(-sigx1*dt/eps))*((Hy(nx-j+1,:)-Hy(nx-j,:))/dx)/sigx1
              ! update Ezy in top and bottom regions
              sigy1=sig2(pml-j+1)
              Ezy(:,j)=exp(-sigy1*dt/eps)*Ezy(:,j)-(1.0d0-exp(-sigy1*dt/eps))*((Hx(:,j+1)-Hx(:,j))/dy)/sigy1
              Ezy(:,ny-j)=exp(-sigy1*dt/eps)*Ezy(:,ny-j)-(1.0d0-exp(-sigy1*dt/eps))*((Hx(:,ny-j+1)-Hx(:,ny-j))/dy)/sigy1
           end do
           Ez(:,:)=Ezx(:,:)+Ezy(:,:)
           ! update source
           if (sourceType==' Gaussian ') then
              Ezx(nx/2,ny/2)=Ezx(nx/2,ny/2)+500.0d0*exp(-(i-t_0)**2/tw**2)
              Ezy(nx/2,ny/2)=Ezy(nx/2,ny/2)+500.0d0*exp(-(i-t_0)**2/tw**2)
           else if (sourceType==' Harmonic ') then
              Ezx(nx/2,ny/2)=Ezx(nx/2,ny/2)+500.0d0*cos(omega*i*dt)
              Ezy(nx/2,ny/2)=Ezy(nx/2,ny/2)+500.0d0*cos(omega*i*dt)
           else
              print*,'Wrong source type!'
           end if
           Et(i)=Ez(nx/2,ny/2)
        end do

        total = etime(elapsed)
        print*,'time elapsed=',total,'sec'

        if ( expNo < 10 ) then
           write( expNoStr, '(i1)' )  expNo
           expNoStr = '00'//expNoStr
        else if ( expNo >= 10 .and. expNo <100 ) then
           write( expNoStr, '(i2)' )  expNo
           expNoStr = '0'//expNoStr
        else
           write( expNoStr, '(i3)' )  expNo
        end if
        
        !fileName = 'SF_ComputingTime.txt'
        !open(unit=0,file=fileName, access='append')
        !write(0, *) nt, total, elapsed(1), elapsed(2)  
        !close(0)  
        
			fileName = 'RefSF_Exp'//expNoStr//'Ez.txt'
			open(unit=1,file=fileName) 	
			do, j=201-nxl/2,200+nyl/2-1
				write (1,"(10000(1x,f0.10))") (Ez(i,j),i=201-nxl/2,200+nyl/2-1)
			end do
			close(1)   
        
        !fileName = 'SF_Exp'//expNoStr//'Hx.txt'
        !open(unit=2,file=fileName) 
        !do, j=1,ny
        !   write (2,"(1000(1x,f0.6))") Hx(:,j)
        !end do
        !close(2)  
        
        !fileName = 'SF_Exp'//expNoStr//'Hy.txt'
        !open(unit=3,file=fileName) 	
        !do, i=1,ny-1
        !   write (3,"(1000(1x,f0.6))") Hy(:,j)
        !end do
        !close(3) 
        
        !fileName = 'SF_Exp'//expNoStr//'Et.txt'
        !open(unit=5,file=fileName)
        !write (5,"(1000(1x,f0.6))") Et(:)
        !close(5) 
                        
        deallocate(Et,Ez,Ezx,Ezy,Hx,Hy,sig1,sig2) 
	
     else
        continueRead = 0
     end if
     
  end do
 
end program splitfield_pml
