!	UniaxialPML.f90 
!
!	FUNCTIONS:
!		UniaxialPML - Implement Gedney's uniaxial Perfectly Matched Layer.
!
!		In various PML profiles and region sizes.
!

!****************************************************************************
!
!	PROGRAM:	UniaxialPML
!
!	PURPOSE:	Implement Gedney's uniaxial PML.
!	
!	AUTHOR:		Chicheng Xu, UT Austin PGE (xuchicheng@mail.utexas.edu)
!
!	DATE:		Mar 19, 2009
!
!****************************************************************************

	program UniaxialPML

	implicit none

	!define math & physics constants
	double precision, parameter :: pi	= 4.0d0*atan(1.0d0)
	double precision, parameter :: u0   = 4.0e-7 * pi	!vacuum permeability
	double precision, parameter :: c0   = 299792458d0	!speed of light in vacuum
	double precision, parameter :: eps0 = 1.0d0/(u0*(c0**2))
	double precision, parameter :: epsr = 1.0d0			!relative permitivity
	double precision, parameter :: ur	= 1.0d0			!relative permeability
	double precision, parameter :: eps = eps0 * epsr
	double precision, parameter :: u = u0 * ur
	double precision, parameter :: twoEps0 = 2.0d0*eps0

	!allocate memory for E and H fields, E at the cross point
	double precision, allocatable :: Et(:)				!sample Ez in one fixed point
	double precision, allocatable :: Ez(:,:)	
	double precision, allocatable :: Hx(:,:)
	double precision, allocatable :: Hy(:,:)
	
	!auxilliary fields for two-step methods
	double precision, allocatable :: Dz(:,:)
	double precision, allocatable :: Bx(:,:)
	double precision, allocatable :: By(:,:)	 
	double precision, allocatable :: Dz0(:,:)
	double precision, allocatable :: Bx0(:,:)
	double precision, allocatable :: By0(:,:)
	double precision, allocatable :: sigma(:)	

	!initialize computing parameters
	double precision lx, ly, m, dt_factor
	integer nx, ny, nt, npml
	double precision dx, dy, dt, t0, tw, omega, sigmaMax, sigmaX, sigmaY
	integer nx0, ny0, n, i, j, expNo
	double precision c1, c2, c3, c4, c5, c6, c7, c8	!common coefficients
	real etime          ! Declare the type of etime()
	real elapsed(2)		!for receiving user and system time
    real total          !for receiving total time

	real t_start, t_stop
	character(len=10)  sourceType
	character(len=30) fileName
	character(len=3) expNoStr
	integer continueRead

	!read computing parameters from external file
	open(unit=100, file = 'CompSettings.txt')

	!skip the first head line
	read(unit=100, fmt='(1X,A10)') sourceType

	expNo = 0
	continueRead = 1

	do while (continueRead == 1)		
		read(unit=100, fmt='(1X,A10,F10.2,F10.2,I10,I10,I10,I10,F10.2,F10.2)') &
		sourceType, lx, ly, nx, ny, nt, npml, m, dt_factor
		
		if ( nx /= 0 )	then	!not reach the end of experiments
			call cpu_time  ( t_start)
			
			expNo = expNo + 1

			nx0 = nx/2 + 1		!center point of x direction
			ny0 = ny/2 + 1		!center point of y direction
			dx = lx / nx        !grid size along x
			dy = ly / ny		!grid size along y
			dt = dt_factor * sqrt(dx**2 + dy**2)/(2.0d0*c0) !magic time step * factor
			omega = pi/(20.0*dt)		!angular frequency of time harmonic source
			
			!max sigma in PML, see Eqn. 38 in Gedney's paper 
			sigmaMax = 90
			!sigmaMax = (m+1)/(150.0d0*pi*dx*sqrt(epsr))			

			!allocate array dimensions
			allocate(Et(nt))
			allocate(Ez(nx+1,ny+1))
			allocate(Hx(nx+1,ny))
			allocate(Hy(nx,ny+1))
			allocate(Dz(nx+1,ny+1))
			allocate(Bx(nx+1,ny))
			allocate(By(nx,ny+1))
			allocate(Dz0(nx+1,ny+1))
			allocate(Bx0(nx+1,ny))
			allocate(By0(nx,ny+1))
			allocate(sigma(2*npml+1))															

			!initialize all fields as zero
			Et = 0
			Ez = 0
			Hx = 0
			Hy = 0
			Dz = 0
			Bx = 0
			By = 0
			Dz0 = 0
			Bx0 = 0
			By0 = 0
			sigma = 0
			
			!initiate the guassian pulse current source
			t0 = 60.0d0           ! The center of the Gaussian Pulse in term of dt
			tw = 10.0d0           ! The width of the Gaussian Pulse in term of dt

			n = 0
			
			!figure source type to use
			if (sourceType == ' Gaussian ') then
				Ez(nx0,ny0) = 1000.0d0 * exp(-(n-t0)**2/(tw**2)) !(dt/eps0)*J(nx0,ny0) at t=0
			else if (sourceType == ' Harmonic ') then
				Ez(nx0,ny0) = 1000.0d0 * cos(omega*n*dt); !(dt/eps0)*J(nx0,ny0) at t=0
			else
				print *, 'Wrong source type!'
			end if

			
			!calculate the conductivities in PML, same profile for x & y directions
			do i=1, 2*npml+1
				sigma(i) = sigmaMax * ((i-1)/(2.0d0*npml))**m
			end do

			!calculate some common coefficients to enhance performance
			c1 = dt/(u0*dy)
			c2 = dt/(u0*dx)
			c3 = dt/(eps0*dy)
			c4 = dt/(eps0*dx)
			c5 = dt/dy
			c6 = dt/dx
			c7 = twoEps0*dt/(u0*ur*dy)
			c8 = twoEps0*dt/(u0*ur*dx)

			!iterate time step
			do n=1,nt
				!********************************************************************
				!update Hx in vacuum space, including the vacuum-PML interface
				do j=npml+1, ny-npml
					Hx(npml+1:nx-npml, j) = Hx(npml+1:nx-npml, j) -					&
					c1*(Ez(npml+1:nx-npml,j+1)-Ez(npml+1:nx-npml,j))
				end do

				!update Hx at the top PML region I, including 
				!the corner-region interface
				do j=ny-npml+1,ny
					sigmaY = sigma(2*(j-ny+npml))
					
					Hx(npml+1:nx-npml,j) = &
					((twoEps0-sigmaY*dt)/(twoEps0+sigmaY*dt))*Hx(npml+1:nx-npml,j) -	&
					(c7/(twoEps0+dt*sigmaY))*(Ez(npml+1:nx-npml,j+1)-Ez(npml+1:nx-npml,j))		
				end do

				!update Hx at the bottom PML region I
				do j=1,npml
					sigmaY = sigma(2*(npml+1-j))
					
					Hx(npml+1:nx-npml,j) = &
					((twoEps0-sigmaY*dt)/(twoEps0+sigmaY*dt))*Hx(npml+1:nx-npml,j) -	&
					(c7/(twoEps0+dt*sigmaY))*(Ez(npml+1:nx-npml,j+1)-Ez(npml+1:nx-npml,j))		
				end do
						
				!update Hx at the left PML region II
				!save current Bx and get the next Bx
				do j=npml+1, ny-npml
					Bx(1:npml, j) = Bx(1:npml, j)-c5*(Ez(1:npml,j+1)-Ez(1:npml,j))
				end do

				do i=1, npml
					sigmaX = sigma(2*(npml+1-i)+1)

					Hx(i,npml+1:ny-npml) = Hx(i,npml+1:ny-npml) +					&
					(1/u)*(Bx(i,npml+1:ny-npml)*(1+(sigmaX*dt)/(twoEps0))-			&
					Bx0(i,npml+1:ny-npml)*(1-(sigmaX*dt)/(twoEps0)))
										
					Bx0(i,npml+1:ny-npml) = Bx(i,npml+1:ny-npml)
				end do		
				
				!update Hx at the right PML region II
				do j=npml+1, ny-npml
					Bx(nx-npml+1:nx+1, j) = Bx(nx-npml+1:nx+1, j)-					&
					c5*(Ez(nx-npml+1:nx+1,j+1)-Ez(nx-npml+1:nx+1,j))
				end do

				do i=nx-npml+1,nx+1
					sigmaX = sigma(2*(i-nx+npml)-1)

					Hx(i,npml+1:ny-npml) = Hx(i,npml+1:ny-npml) + &
					(1/u)*(Bx(i,npml+1:ny-npml)*(1+(sigmaX*dt)/(twoEps0))-			&
					Bx0(i,npml+1:ny-npml)*(1-(sigmaX*dt)/(twoEps0)))

					Bx0(i,npml+1:ny-npml) = Bx(i,npml+1:ny-npml)
				end do
				
				!update Hx at the bottom-left PML region III
				do j=1,npml
					sigmaY = sigma(2*(npml+1-j))			

					Bx(1:npml,j) =														&
					((twoEps0-sigmaY*dt)/(twoEps0+sigmaY*dt))*Bx(1:npml,j) -			&
					(c7/(twoEps0+dt*sigmaY))*(Ez(1:npml,j+1)-Ez(1:npml,j))	
				end do
					
				do i=1, npml
					sigmaX = sigma(2*(npml+1-i)+1)

					Hx(i,1:npml) = Hx(i,1:npml) +										&
					(Bx(i,1:npml)*(1+(sigmaX*dt)/(twoEps0))-							&
					Bx0(i,1:npml)*(1-(sigmaX*dt)/(twoEps0)))

					Bx0(i,1:npml) = Bx(i,1:npml)
				end do
						
				!update Hx at the bottom-right PML region III
				do j=1,npml
					sigmaY = sigma(2*(npml+1-j))			
			
					Bx(nx-npml+1:nx+1,j) =												&
					((twoEps0-sigmaY*dt)/(twoEps0+sigmaY*dt))*Bx(nx-npml+1:nx+1,j) -	&
					(c7/(twoEps0+dt*sigmaY))*(Ez(nx-npml+1:nx+1,j+1)-Ez(nx-npml+1:nx+1,j))	
				end do
					
				do i=nx-npml+1,nx+1
					sigmaX = sigma(2*(i-nx+npml)-1)

					Hx(i,1:npml) = Hx(i,1:npml) + &
					(Bx(i,1:npml)*(1+(sigmaX*dt)/(twoEps0))-							&
					Bx0(i,1:npml)*(1-(sigmaX*dt)/(twoEps0)))

					Bx0(i,1:npml) = Bx(i,1:npml)
				end do				
				
				!update Hx at the top-left PML region III
				do j=ny-npml+1,ny
					sigmaY = sigma(2*(j-ny+npml))		

					Bx(1:npml,j) =														&
					((twoEps0-sigmaY*dt)/(twoEps0+sigmaY*dt))*Bx(1:npml,j) -			&
					(c7/(twoEps0+dt*sigmaY))*(Ez(1:npml,j+1)-Ez(1:npml,j))	
				end do
					
				do i=1, npml
					sigmaX = sigma(2*(npml+1-i)+1)

					Hx(i,ny-npml+1:ny) = Hx(i,ny-npml+1:ny) + &
					(Bx(i,ny-npml+1:ny)*(1+(sigmaX*dt)/(twoEps0))-						&
					Bx0(i,ny-npml+1:ny)*(1-(sigmaX*dt)/(twoEps0)))

					Bx0(i,ny-npml+1:ny) = Bx(i,ny-npml+1:ny)
				end do
				
				!update Hx at the top-right PML region III
				do j=ny-npml+1,ny					
					sigmaY = sigma(2*(j-ny+npml))			

					Bx(nx-npml+1:nx+1,j) =												&
					((twoEps0-sigmaY*dt)/(twoEps0+sigmaY*dt))*Bx(nx-npml+1:nx+1,j) -	&
					(c7/(twoEps0+dt*sigmaY))*(Ez(nx-npml+1:nx+1,j+1)-Ez(nx-npml+1:nx+1,j))	
				end do
					
				do i=nx-npml+1,nx+1
					sigmaX = sigma(2*(i-nx+npml)-1)

					Hx(i,ny-npml+1:ny) = Hx(i,ny-npml+1:ny) + &
					(Bx(i,ny-npml+1:ny)*(1+(sigmaX*dt)/(twoEps0))-						&
					Bx0(i,ny-npml+1:ny)*(1-(sigmaX*dt)/(twoEps0)))

					Bx0(i,ny-npml+1:ny) = Bx(i,ny-npml+1:ny)
				end do
		
				!Due to PEC frame
				Hx(1,:)=0
				Hx(nx+1,:)=0
				!********************************************************************

				!********************************************************************
				!update Hy in vacuum space where sigma = 0
				do i=npml+1, nx-npml
					Hy(i, npml+1:ny-npml) = Hy(i, npml+1:ny-npml) +					&
					c2*(Ez(i+1,npml+1:ny-npml)-Ez(i,npml+1:ny-npml))
				end do

				!update Hy at the top PML region I
				do i=npml+1, nx-npml
					By(i, ny-npml+1:ny+1) = By(i, ny-npml+1:ny+1) +					&
					c6*(Ez(i+1, ny-npml+1:ny+1)-Ez(i, ny-npml+1:ny+1))
				end do

				do j=ny-npml+1,ny+1
					sigmaY = sigma(2*(j-ny+npml)-1)

					Hy(npml+1:nx-npml,j) = Hy(npml+1:nx-npml,j) + &
					(1/u)*(By(npml+1:nx-npml,j)*(1+(sigmaY*dt)/(twoEps0))-			&
					By0(npml+1:nx-npml,j)*(1-(sigmaY*dt)/(twoEps0)))

					By0(npml+1:nx-npml,j) = By(npml+1:nx-npml,j)
				end do

				!update Hy at the bottom PML region I
				do i=npml+1, nx-npml
					By(i, 1:npml) = By(i, 1:npml) + c6*(Ez(i+1, 1:npml)-Ez(i, 1:npml))
				end do

				do j=1, npml
					sigmaY = sigma(2*(npml+1-j)+1)

					Hy(npml+1:nx-npml,j) = Hy(npml+1:nx-npml,j) +					&
					(1/u)*(By(npml+1:nx-npml,j)*(1+(sigmaY*dt)/(twoEps0))-			&
					By0(npml+1:nx-npml,j)*(1-(sigmaY*dt)/(twoEps0)))

					By0(npml+1:nx-npml,j) = By(npml+1:nx-npml,j)
				end do	
				
				!update Hy at the left PML region II
				do i=1,npml
					sigmaX = sigma(2*(npml+1-i))
					
					Hy(i, npml+1:ny-npml) =											&
					((twoEps0-sigmaX*dt)/(twoEps0+sigmaX*dt))*Hy(i, npml+1:ny-npml) + &
					(c8/(twoEps0+dt*sigmaX))*(Ez(i+1, npml+1:ny-npml)-Ez(i, npml+1:ny-npml))		
				end do

				!update Hy at the right PML region II
				do i=nx-npml+1,nx
					sigmaX = sigma(2*(i-nx+npml))
					
					Hy(i, npml+1:ny-npml) =											&
					((twoEps0-sigmaX*dt)/(twoEps0+sigmaX*dt))*Hy(i, npml+1:ny-npml) + &
					(c8/(twoEps0+dt*sigmaX))*(Ez(i+1, npml+1:ny-npml)-Ez(i, npml+1:ny-npml))		
				end do

				!update Hy at the bottom-left PML region III
				do i=1,npml
					sigmaX = sigma(2*(npml+1-i))			

					By(i, 1:npml) =													&
					((twoEps0-sigmaX*dt)/(twoEps0+sigmaX*dt))*By(i, 1:npml) +			&
					(c8/(twoEps0+dt*sigmaX))*(Ez(i+1, 1:npml)-Ez(i, 1:npml))	
				end do
					
				do j=1, npml
					sigmaY = sigma(2*(npml+1-j)+1)

					Hy(1:npml,j) = Hy(1:npml,j) + &
					(By(1:npml,j)*(1+(sigmaY*dt)/(twoEps0))-							&
					By0(1:npml,j)*(1-(sigmaY*dt)/(twoEps0)))

					By0(1:npml,j) = By(1:npml,j)
				end do

				!update Hy at the bottom-right PML region III
				do i=nx-npml+1,nx
					sigmaX = sigma(2*(i-nx+npml))			

					By(i, 1:npml) =													&
					((twoEps0-sigmaX*dt)/(twoEps0+sigmaX*dt))*By(i,1:npml) +			&
					(c8/(twoEps0+dt*sigmaX))*(Ez(i+1, 1:npml)-Ez(i, 1:npml))	
				end do
					
				do j=1, npml
					sigmaY = sigma(2*(npml+1-j)+1)

					Hy(nx-npml+1:nx,j) = Hy(nx-npml+1:nx,j) +						&
					(By(nx-npml+1:nx,j)*(1+(sigmaY*dt)/(twoEps0))-					&
					By0(nx-npml+1:nx,j)*(1-(sigmaY*dt)/(twoEps0)))

					By0(nx-npml+1:nx,j) = By(nx-npml+1:nx,j)
				end do				
				
				!update Hy at the top-left PML region III
				do i=1,npml
					sigmaX = sigma(2*(npml+1-i))			

					By(i, ny-npml+1:ny+1) =											&
					((twoEps0-sigmaX*dt)/(twoEps0+sigmaX*dt))*By(i, ny-npml+1:ny+1) + &
					(c8/(twoEps0+dt*sigmaX))*(Ez(i+1, ny-npml+1:ny+1)-Ez(i, ny-npml+1:ny+1))	
				end do
					
				do j=ny-npml+1,ny+1
					sigmaY = sigma(2*(j-ny+npml)-1)

					Hy(1:npml,j) = Hy(1:npml,j) +									&
					(By(1:npml,j)*(1+(sigmaY*dt)/(twoEps0))-							&
					By0(1:npml,j)*(1-(sigmaY*dt)/(twoEps0)))

					By0(1:npml,j) = By(1:npml,j)
				end do

				!update Hy at the top-right PML region III
				do i=nx-npml+1,nx
					sigmaX = sigma(2*(i-nx+npml))			

					By(i, ny-npml+1:ny+1) =												&
					((twoEps0-sigmaX*dt)/(twoEps0+sigmaX*dt))*By(i, ny-npml+1:ny+1) +	&
					(c8/(twoEps0+dt*sigmaX))*(Ez(i+1, ny-npml+1:ny+1)-Ez(i, ny-npml+1:ny+1))	
				end do
					
				do j=ny-npml+1,ny+1
					sigmaY = sigma(2*(j-nx+npml)-1)

					Hy(nx-npml+1:nx,j) = Hy(nx-npml+1:nx,j) +						&
					(By(nx-npml+1:nx,j)*(1+(sigmaY*dt)/(twoEps0))-					&
					By0(nx-npml+1:nx,j)*(1-(sigmaY*dt)/(twoEps0)))
					
					By0(nx-npml+1:nx,j) = By(nx-npml+1:nx,j)				
				end do

				!due to PEC frame
				Hy(:, 1)=0
				Hy(:, ny+1)=0
				!********************************************************************

				!********************************************************************
				!update Ez in vacuum space without current source
				do i=npml+1, nx+1-npml
					do j = npml+1, ny+1-npml
						Ez(i,j) = Ez(i,j)+ c4*(Hy(i,j)-Hy(i-1,j)) -	c3*(Hx(i,j)-Hx(i,j-1))
					end do
				end do

				!apply source in the center; source term = (dt/eps0)*J(nx0,ny0, t)			
				if (sourceType == ' Gaussian ') then
					Ez(nx0,ny0) = Ez(nx0,ny0) + 1000*exp(-(n-t0)**2/tw**2); 					
				else if (sourceType == ' Harmonic ') then
					Ez(nx0,ny0) = Ez(nx0,ny0) + 1000.0 * cos(omega*n*dt);					
				else
					print *, 'Wrong source type!'
				end if				
				
				
				!update Ez at the top PML region I		
				do j=ny-npml+2,ny
					sigmaY = sigma(2*(j-ny+npml)-1)
						
					do i=npml+1, nx-npml+1				
						Ez(i,j) = ((twoEps0-sigmaY*dt)/(twoEps0+sigmaY*dt))*Ez(i,j) +	&
						(2*dt/(epsr*(twoEps0+dt*sigmaY)))*								&
						((Hy(i,j)-Hy(i-1,j))/dx-(Hx(i,j)-Hx(i,j-1))/dy)		
					end do
				end do

				!update Ez at the bottom PML region I
				do j=2,npml
					sigmaY = sigma(2*(npml+1-j)+1)
						
					do i=npml+1, nx-npml+1				
						Ez(i,j) = ((twoEps0-sigmaY*dt)/(twoEps0+sigmaY*dt))*Ez(i,j) +	&
						(2*dt/(epsr*(twoEps0+dt*sigmaY)))*								&	
						((Hy(i,j)-Hy(i-1,j))/dx-(Hx(i,j)-Hx(i,j-1))/dy)		
					end do
				end do

				!update Ez at the left PML region II
				do i=2,npml
					sigmaX = sigma(2*(npml+1-i)+1)
						
					do j=npml+1, ny-npml+1				
						Ez(i,j) = ((twoEps0-sigmaX*dt)/(twoEps0+sigmaX*dt))*Ez(i,j) +	&
						(2*dt/(epsr*(twoEps0+dt*sigmaX)))*								&
						((Hy(i,j)-Hy(i-1,j))/dx-(Hx(i,j)-Hx(i,j-1))/dy)		
					end do
				end do

				!update Ez at the right PML region II
				do i=nx-npml+2,nx
					sigmaX = sigma(2*(i-nx+npml)-1)
						
					do j=npml+1, ny-npml+1				
						Ez(i,j) = ((twoEps0-sigmaX*dt)/(twoEps0+sigmaX*dt))*Ez(i,j) +	&
						(2*dt/(epsr*(twoEps0+dt*sigmaX)))*								&
						((Hy(i,j)-Hy(i-1,j))/dx-(Hx(i,j)-Hx(i,j-1))/dy)		
					end do
				end do
				
				!update Ez at the top-left PML region III
				!get Dz
				do i=2,npml
					sigmaX = sigma(2*(npml+1-i)+1)
						
					do j=ny-npml+2, ny				
						Dz(i,j) = ((twoEps0-sigmaX*dt)/(twoEps0+sigmaX*dt))*Dz(i,j) + &
						(2*dt/(epsr*(twoEps0+dt*sigmaX)))*							&
						((Hy(i,j)-Hy(i-1,j))/dx-(Hx(i,j)-Hx(i,j-1))/dy)		
					end do
				end do

				do j=ny-npml+2, ny
					sigmaY = sigma(2*(j-ny+npml)-1)

					Ez(2:npml, j)=												&
					((twoEps0-sigmaY*dt)/(twoEps0+sigmaY*dt))*Ez(2:npml, j)+	&
					twoEps0*(Dz(2:npml, j)-Dz0(2:npml, j))/(twoEps0+sigmaY*dt)

					Dz0(2:npml, j) = Dz(2:npml, j)
				end do

				!update Ez at the top-right PML region III
				!get Dz
				do i=nx-npml+2,nx
					sigmaX = sigma(2*(i-nx+npml)-1)
						
					do j=ny-npml+2, ny				
						Dz(i,j) = ((twoEps0-sigmaX*dt)/(twoEps0+sigmaX*dt))*Dz(i,j) + &
						(2*dt/(epsr*(twoEps0+dt*sigmaX)))*							&
						((Hy(i,j)-Hy(i-1,j))/dx-(Hx(i,j)-Hx(i,j-1))/dy)		
					end do
				end do

				do j=ny-npml+2, ny
					sigmaY = sigma(2*(j-ny+npml)-1)

					Ez(nx-npml+2:nx, j)=											&
					((twoEps0-sigmaY*dt)/(twoEps0+sigmaY*dt))*Ez(nx-npml+2:nx, j)+	&
					twoEps0*(Dz(nx-npml+2:nx, j)-Dz0(nx-npml+2:nx, j))/(twoEps0+sigmaY*dt)

					Dz0(nx-npml+2:nx, j) = Dz(nx-npml+2:nx, j)
				end do


				!update Ez at the bottom-left PML region III
				!get Dz
				do i=2,npml
					sigmaX = sigma(2*(npml+1-i)+1)
						
					do j=2, npml				
						Dz(i,j) = ((twoEps0-sigmaX*dt)/(twoEps0+sigmaX*dt))*Dz(i,j) + &
						(2*dt/(epsr*(twoEps0+dt*sigmaX)))*							&
						((Hy(i,j)-Hy(i-1,j))/dx-(Hx(i,j)-Hx(i,j-1))/dy)		
					end do
				end do

				do j=2, npml
					sigmaY = sigma(2*(npml+1-j)+1)

					Ez(2:npml, j)=													&
					((twoEps0-sigmaY*dt)/(twoEps0+sigmaY*dt))*Ez(2:npml, j)+			&
					twoEps0*(Dz(2:npml, j)-Dz0(2:npml, j))/(twoEps0+sigmaY*dt)

					Dz0(2:npml, j) = Dz(2:npml, j)
				end do

				!update Ez at the bottom-right PML region III
				!get Dz
				do i=nx-npml+2,nx
					sigmaX = sigma(2*(i-nx+npml)-1)
						
					do j=2, npml				
						Dz(i,j) = ((twoEps0-sigmaX*dt)/(twoEps0+sigmaX*dt))*Dz(i,j) + &
						(2*dt/(epsr*(twoEps0+dt*sigmaX)))*							&
						((Hy(i,j)-Hy(i-1,j))/dx-(Hx(i,j)-Hx(i,j-1))/dy)		
					end do
				end do

				do j=2, npml
					sigmaY = sigma(2*(npml+1-j)+1)

					Ez(nx-npml+2:nx, j)=											&
					((twoEps0-sigmaY*dt)/(twoEps0+sigmaY*dt))*Ez(nx-npml+2:nx, j)+	&
					twoEps0*(Dz(nx-npml+2:nx, j)-Dz0(nx-npml+2:nx, j))/(twoEps0+sigmaY*dt)

					Dz0(nx-npml+2:nx, j) = Dz(nx-npml+2:nx, j)
				end do

				!due to PEC frame
				Ez(1,:)=0
				Ez(nx+1,:)=0
				Ez(:,1)=0
				Ez(:,ny+1)=0

				!sample the electric field at chosen point
				Et(n) = Ez(nx0+10,ny0+1);
				!********************************************************************
			end do

			!get elapse time
			call cpu_time ( t_stop )
			print *, 'Elapsed CPU time = ', t_stop - t_start

			!convert the experiment no. to string and use it in the output file name
			if ( expNo < 10 ) then
				write( expNoStr, '(i1)' )  expNo
				expNoStr = '00'//expNoStr
			else if ( expNo >= 10 .and. expNo <100 ) then
				write( expNoStr, '(i2)' )  expNo
				expNoStr = '0'//expNoStr
			else
				write( expNoStr, '(i3)' )  expNo
			end if
			
		total = etime(elapsed)




	        fileName = 'UX_ComputingTime.txt'
        open(unit=0,file=fileName, access='append')
        write(0, *) elapsed(2)  
        close(0) 
		
			fileName = 'UX_Exp'//expNoStr//'Ez.txt'
			open(unit=1,file=fileName) 	
			do, j=1,ny+1
				write (1,"(10000(1x,f0.10))") Ez(:,j)
			end do
			close(1)  



			deallocate( Et, Ez, Hx, Hy, Dz, Bx, By, Dz0, Bx0, By0, sigma ) 
	
		else
			continueRead = 0
		end if 

	end do

	close(100)

	end program UniaxialPML